#include "../../include/error.h"
#include "../../include/util.h"
#include "../../include/pool.h"
#include "../../include/pkg.h"

#include <stdlib.h>
#include <string.h>

lm_pool_t *lm_pool_new(char *name, char *url) {
  lm_pool_t *pool = malloc(sizeof(lm_pool_t));
  bzero(pool, sizeof(lm_pool_t));
  
  pool->available = true;
  pool->name      = name;

  if (!lm_url_init(&pool->url, url)) {
    free(pool);
    return NULL;
  }

  if (!eq(pool->url.protocol, "mptp")) {
    lm_error_set(LM_ERR_PoolNoSupport);
    lm_pool_free(pool);
    return NULL;
  }

  return pool;
}

void lm_pool_test(lm_pool_t *pool) {
  lm_mptp_t packet;
  lm_mptp_init(&packet, true, MPTP_C2S_PING, true);

  lm_mptp_set_host(&packet, pool->url.host);
  lm_mptp_set_data(&packet, pool->url.path, strlen(pool->url.path));

  int sock = lm_mptp_client_connect(pool->url.host, pool->url.port);
  if (sock == -1) {
    pool->available = false;
    return;
  }

  if (!lm_mptp_client_send(sock, &packet)) {
    pool->available = false;
    goto end;
  }

  if (!lm_mptp_client_recv(sock, &packet)) {
    pool->available = false;
    goto end;
  }

  if (!lm_mptp_client_verify(&packet)) {
    pool->available = false;
    goto end;
  }

  pool->available = MPTP_FLAGS_CODE(&packet) == MPTP_S2C_PONG;
end:
  lm_mptp_close(sock);
  return;
}

bool lm_pool_add(lm_pool_t *pool, lm_pkg_t *pkg){
  if(NULL == pool || NULL == pkg){
    lm_error_set(LM_ERR_ArgNULL);
    return false;
  }

  if(NULL == pool->pkg){
    pool->pkg = pkg;
    return true;
  }

  lm_pkg_t *cur = pool->pkg;
  while(NULL != cur){
    if(NULL == cur->next){
      cur->next = pkg;
      return true;
    }
    cur = cur->next;
  }

  return false;
}

void lm_pool_free(lm_pool_t *pool) {
  lm_url_free(&pool->url);
  lm_pool_info_free(pool);

  lm_pkg_t *cur = pool->pkg, *prev = NULL;
  while(NULL != cur){
    prev = cur;
    cur = cur->next;
    lm_pkg_free(prev);
  }

  free(pool);
}
